# How to contribute

One of the easiest ways to contribute is to participate in discussions on issues. You can also contribute by submitting pull requests with code changes if you have been invited. Invitations are send to exemplary members of the SmartAdmin for ASP.NET Core community and/or those who have explicitly expressed in writing to be part of contributing to the further development of SmartAdmin for ASP.NET Core.

## General feedback and discussions

Start a discussion on the [repository issue tracker](https://bitbucket.org/smartadmin-next/smartadmincore/issues).

## Bugs and feature requests

For non-security related bugs, log a new issue in the appropriate repository. Here are some of the most common repositories:

* [SmartAdmin Support Forum](https://support.gotbootstrap.com/t/asp-net-core)
* [SmartAdminCore](https://bitbucket.org/smartadmin-next/smartadmincore)

Or browse the full list of repositories in the [smartadmin](https://bitbucket.org/smartadmin-next/) organization.

## Reporting security issues and bugs

Security issues and bugs should be reported privately, via email, to SmartAdmin Security Response Center (MSRC) <secure@walapa.nl>. You should receive a response within 24 hours. If for some reason you do not, please follow up via email to ensure we received your original message.

## Other discussions

Our team members also monitor several other discussion forums:

* [SmartAdmin Support Forum](https://support.gotbootstrap.com/t/asp-net-core)
* [WrapBootstrap Comments](https://wrapbootstrap.com/theme/smartadmin-responsive-webapp-WB0573SK0/comments)
* [Stack Overflow](https://stackoverflow.com/) with the [`smartadmin-aspnetcore`](https://stackoverflow.com/questions/tagged/smartadmin-aspnetcore) or [`smartadmin-dotnet`](https://stackoverflow.com/questions/tagged/smartadmin-next-dotnet) tags.

## Contributing code and content

### Identifying the scale

If you would like to contribute to one of our repositories, first identify the scale of what you would like to contribute. If it is small (grammar/spelling or a bug fix) feel free to start working on a fix. If you are submitting a feature or substantial code contribution, please discuss it with the team and ensure it follows the product roadmap. You might also read these two blogs posts on contributing code: [Open Source Contribution Etiquette](http://tirania.org/blog/archive/2010/Dec-31.html) by Miguel de Icaza and [Don't "Push" Your Pull Requests](https://www.igvita.com/2011/12/19/dont-push-your-pull-requests/) by Ilya Grigorik. All code submissions will be rigorously reviewed and tested by the SmartAdmin ASP.NET team, and only those that meet an extremely high bar for both quality and design/roadmap appropriateness will be merged into the source.

### Submitting a pull request

You will need to submit your changes as a pull request. If you don't know what a pull request is read this article: <https://help.github.com/articles/using-pull-requests>. Make sure the repository can build and all tests pass. Familiarize yourself with the project workflow and our coding conventions. The coding, style, and general engineering guidelines are published on the [Engineering guidelines](https://github.com/aspnet/AspNetCore/wiki/Engineering-guidelines) page.

### Tests

* Tests need to be provided for every bug/feature that is completed.
* Tests only need to be present for issues that need to be verified by QA (for example, not tasks)
* If there is a scenario that is far too hard to test there does not need to be a test for it.
  * "Too hard" is determined by the team as a whole.

### Feedback

Your pull request will now go through extensive checks by the subject matter experts on our team. Please be patient; we have a multitude of pull requests across all of our repositories. Update your pull-request according to feedback until it is approved by one of the team members. After that, one of our team members may adjust the branch you merge into based on the expected release schedule.

## Code of conduct

This project has adopted the [Microsoft Open Source Code of Conduct](https://opensource.microsoft.com/codeofconduct/).  For more information, see the [Code of Conduct FAQ](https://opensource.microsoft.com/codeofconduct/faq/) or contact [smartadmin-next@walapa.nl](mailto:smartadmin-next@walapa.nl) with any additional questions or comments.