# SmartAdmin for ASP.NET Core 3.1 - Documentation

## Table of Contents

1. **Introduction**
1. **[Getting Started](getting-started.md)**
1. **[Site Structure](site-structure.md)**
1. **[Solution Architecture](solution-architecture.md)**
1. **[Customizations](customizations.md)**
1. **[How To Contribute](howto-contribute.md)**
1. **[Licensing Information](licensing-information.md)**
1. **[Changelog](changelog.md)**

---

### Thank You

We are excited to have you as our customer and as with any Theme and/or Product, good documentation is essential to provide you with that first solid impression and we hope we have delivered on that expectation.

These pages should allow you to get a good and basic understanding of what the template has to offer, in addition it should provide you with the necessary information to understand how the Project and Theme have been structured for use with ASP.NET.

Although **SmartAdmin for ASP.NET Core** and the **Documentation** is primarily created with ASP.NET MVC in mind, other technologies such as [Razor Pages](https://docs.microsoft.com/en-us/aspnet/core/tutorials/razor-pages/razor-pages-start) and [Blazor](https://www.devexpress.com/support/training/blazor/) are not be ignored and can safely be used side-by-side if this is part of the requirement for your project.

In the case of the **Documentation** not every example is equally applicable when **Razor Pages** are used, so some time investment might be required to locate the Razor Page equivalent instruction and/or example, our apologies.

> **Note:** Please see the [ASP.NET Flavors](#Flavors) section of the documentation for more information on the available flavors of ASP.NET this theme currently has to offer.

### SmartAdmin for ASP.NET Core

This template and project demonstrates the core principles and concepts of how you can integrate SmartAdmin 4.x with ASP.NET Core. Although we aim, and do our best, to create a reference project based on community adopted guidelines and principles, this project does not however claim to be **the** best way to integrate templates within ASP.NET. That said, with our years of experience and our focus on quality we believe that this template will provide you with a great starting point that you can use for your own project purposes and customizations.

Having a basic level of knowledge about the ASP.NET Platform and the .NET Core Framework is not a must but will certainly help you understand a lot more of what is explained. To get a head start on these topics, we would kindly ask you to check the [Resources](#Resources) listed below.

We will do our best to ensure that all levels of experience can benefit and read along!

## Table of Contents

1. **Introduction**
1. **[Getting Started](getting-started.md)**
1. **[Site Structure](site-structure.md)**
1. **[Solution Architecture](solution-architecture.md)**
1. **[Customizations](customizations.md)**
1. **[How To Contribute](howto-contribute.md)**
1. **[Licensing Information](licensing-information.md)**
1. **[Changelog](changelog.md)**

## Flavors

The package you downloaded is divided into several projects, or **Flavors**, that should align with your project needs, whether it's a quick start of a few pages or a full-blown management portal, each project has its own unique set of features to help you decide.

And if you can't find the flavor you were looking for? No problem, the projects and theme have been structured in such a way that combining any set of features should be achievable with minimal effort.

With that said, here is the overview of the current flavors that are supported in **SmartAdmin for ASP.NET**.

- **SmartAdmin for ASP.NET Core 3.1 - Bare** - Want to start from scratch and slowly add to your site structure and pages? Then **Bare** will be your best starting point.

- **SmartAdmin for ASP.NET Core 3.1 - Slim** - Need a light version with minimal plugins and fast page loading without compromise? No database or authentication support needed? Then **Slim** will be your best starting point.

- **SmartAdmin for ASP.NET Core 3.1 - Seed** - Want to checkout the most popular pages SmartAdmin has to offer, such as Dashboards, Charts, Chat and other cool features? Then **Seed** will allow you to view all the required markup to start planning the features you want to include.

- **SmartAdmin for ASP.NET Core 3.1 - Full** - Planning on building a fully fledged out Management Portal with Dashboards and database driven logic and data combined with other cool features? Then **Full** will ensure that you will have all the necessary examples and plugins to start building your next big project, including a well adopted architecture and trusted design patterns to guide your every step!

> **Note:** For a full matrix overview of the features per Flavor please see the [Editions](https://smartadmin-core.azurewebsites.net/aspnetcore/editions) page, available through the online Preview of the **Full** flavor.

### ASP.NET Razor Pages and Blazor

We have also included a complete **Razor Pages** based flavor of the SmartAdmin Theme using the **Full** feature set. Any and all features, such as Filters and Authentication have been thouroughly tested to ensure these remain compatible when switching to the **Razor Pages** flavor.

> **Note:** We are currently still evaluating the possibility of adding a **server-side** Blazor flavor to the Project, but we want to make sure this does not compromise the feature set of the Theme, so please stay tuned for future updates!

## Resources

The following references can be used to get a basic understanding of the technologies and concepts that are used throughout the **SmartAdmin for ASP.NET Core** Theme:

1. [ASP.NET Introduction](https://dotnet.microsoft.com/learn/aspnet/what-is-aspnet)
1. [MVC Introduction](https://dotnet.microsoft.com/apps/aspnet/mvc)
1. [EF Core Introduction](https://docs.microsoft.com/en-us/ef/core/)
1. [Razor Pages Introduction](https://docs.microsoft.com/en-us/aspnet/core/tutorials/razor-pages/razor-pages-start)
1. [ASP.NET Core Identity](https://docs.microsoft.com/en-us/aspnet/core/security/authentication/identity)
1. [Blazor Introduction](https://www.devexpress.com/support/training/blazor/)
1. [C# Introduction](https://docs.microsoft.com/en-us/dotnet/csharp/getting-started/)
1. [Coding Guidelines](https://docs.microsoft.com/en-us/dotnet/csharp/programming-guide/inside-a-program/coding-conventions)
1. [Best Practices](https://docs.microsoft.com/en-us/aspnet/core/performance/performance-best-practices)
1. [ASP.NET Tutorials](https://dotnet.microsoft.com/learn/aspnet)

---

Copyright &copy; 2020 by Walapa. All rights reserved. This documentation or any portion thereof
may not be reproduced or used in any manner whatsoever without the express written permission of the publisher except for the use of brief quotations in a review.
